// https://observablehq.com/@sengdowjones/opening-hierachy-by-rating-level-on-lichess@784
function _1(md){return(
md`# Chess Opening Hierarchy by Rating Level on [Lichess](https://www.kaggle.com/datasnaek/chess)
**Lichess** is an open source chess server founded in 2010, the second most popular chess playing website after chess.com in the world. Every month, Lichess releases a dataset of all the chess games played on the website. This circle-packing data visualization on this page is made from a snapshot of Lichess's data from 2017.
`
)}

function _2(md){return(
md`## A Brief Introduction to Reading Chess Notation`
)}

function _3(md){return(
md`In order to understand the visualization, it is best to understand how **chess notation** works. If you already are familiar with chess notation, feel free to skip this section. <br> 

A **chess board** is divided into eight rows and eight columns called ranks and files respectively. The files are denoted with letters a-h, starting on the left, and the columns are denoted 1-8, starting on  the bottom. Each tile is then notated with its file and rank, as shown in the chess board. <br>



`
)}

async function _4(FileAttachment,md){return(
md`![chess board.jpg](${await FileAttachment("chess board.jpg").url()})`
)}

function _5(md){return(
md`A **chess game** is recorded in chess notation, specifying which tiles have been moved into and if there was any capturing. The beginning four moves of a chess game might look like this: <br>
*1. d4 1... e6* <br>
*2. e4 2... Ne7* <br>
With the corresponding game shown below.`
)}

async function _6(FileAttachment,md){return(
md`![Screenshot 2022-03-08 004236.png](${await FileAttachment("Screenshot 2022-03-08 004236.png").url()})`
)}

function _7(md){return(
md`The first move is on the white side, and it is a pawn to tile d4. In chess notation, pawns do not get a prefix on the tile number while all other pieces do. So from our example, Ne7 means that the knight has moved to space e7.  If one piece captures another, the notation is *piece name + x + the tile it took*. So if knight takes a pawn on e7, the notation is Nxe7. Since pawns don’t have an abbreviation, their notation starts with the file they came from.  So if a pawn on e4 took a pawn on d5, the notation would be exd5. You can read more about chess notation [here](https://www.chess.com/terms/chess-notation), or play a chess game to see it in action [here](https://www.mathsisfun.com/games/chess.html).`
)}

function _8(md){return(
md`# Opening Hierarchy by Rating Level`
)}

function _9(md){return(
md`The **area** of each leaf circle in a circle-packing diagram is proportional to its value (here, move frequency). 

**The nested circles reveal the hierarchical structure of the first 2 moves by each player for the following rating levels: beginner, intermediate, and advanced.** 

We denote the beginner level as 700-1499, the intermediate level as 1500-1999, and the advanced level as 2000+ based on Lichess rating.

Each move is differentiated by the color of a leaf circle with popular responses by Black such as d5 and c5 in bright shades of yellow.`
)}

function _10(md){return(
md`### Sidenote: What is rating?`
)}

function _11(md){return(
md`Lichess **rating** is a numerical value that measures a player's playing strength. This number fluctuates based on consistent results against players of other rating strengths. A player will gain more rating by a win against an opponent higher-rated than them. The higher one's rating is, the higher their deemed playing strength.`
)}

function _12(md){return(
md`On Lichess, all ratings start at 1500 under what is called the [**Glicko**](https://en.wikipedia.org/wiki/Glicko_rating_system) system.  Below is the general distribution of rating on Lichess which reveals a bell curve (normal distribution). The yellow line is the cumulative density function that represents the percentile of each rating.`
)}

async function _13(FileAttachment,md){return(
md`![Screenshot 2022-03-08 102817.png](${await FileAttachment("Screenshot 2022-03-08 102817.png").url()})`
)}

function _14(md){return(
md`## Beginner (700+)`
)}

function _15(md){return(
md`### Analysis 

At the beginner level, we see a total of 20 unique openings being used. The two most dominant choices to start the game with are equally **1. d4** and **1. e4**. Furthermore, note the quantity of moves to follow-up d4 and e4. There are 20 notable responses to e4 with **1... e5** being the most frequent. **1... d5** is the most popular response to d4.

The **diversity** of opening repertoire at this level is characteristic of beginners who have only begun experimenting with different openings and who have begun to utilize introductory chess opening principles such as controlling the center — represented by the starting center-pawn moves of e4 and d4.

Click on one of the circles below to start exploring the sequence of moves that beginners utilize most often. You can go back in your view by clicking outside the zoomed-in circle and reset your view completely by clicking the black space. Try iterating through the Sicilian Defense **(1. e4 1... c5)** and notice the reigning size of *Bc4* compared to the rest of the pack. We will be comparing this at each rating level.`
)}

function _beginner(pack,beginner_data,d3,width,height,color)
{
  const root = pack(beginner_data);
  let focus = root;
  let view;

  var div = d3.select("body").append("div")
     .style("position", "absolute")
     .style("text-align", "center")
     .style("padding", .5)
     .style("background", "#FFFFFF")
     .style("color", "#313639")
     .style("border", "1px solid #313639")
     .style("border-radius", "8px")
     .style("font-size", "1.3rem")
     .style("opacity", 0);

  const svg = d3.create("svg")
      .attr("viewBox", `-${width / 2} -${height / 2} ${width} ${height}`)
      .style("display", "block")
      .style("margin", "0 -14px")
      .style("background", "black")
      .style("cursor", "pointer")
      .on("click", (event) => zoom(event, root));
  
  const node = svg.append("g")
    .selectAll("circle")
    .data(root.descendants().slice(1))
    .join("circle")
      .attr("fill", d => d.children ? color(d.data.name) : "black")
      .attr("pointer-events", d => !d.children ? "none" : null)
      .on("mouseover", function(d) { d3.select(this).attr("stroke", "#ffffff").attr("stroke-width", 3); 
                                    div.transition().duration('50').style("opacity",1); 
                                    let name = d.data.name;
                                    div.html(name).style("left", (d3.event.pageX + 10) + "px")
                                                  .style("top", (d3.event.pageY - 15) + "px");})
      .on("mouseout", function() { d3.select(this).attr("stroke", null); 
                                   div.transition().duration('50').style("opacity", 0); })
      .on("click", (event, d) => focus !== d && (zoom(event, d), event.stopPropagation()));
    
  node.append("title")
      .text(d => !d.data.children? `${d.data.name}` : `${d.data.name}\nUnique Responses: ${d.data.children.length}`);

  const label = svg.append("g")
      .style("font", "48px sans-serif")
      .attr("fill", "white")
      .attr("pointer-events", "none")
      .attr("text-anchor", "middle")
    .selectAll("text")
    .data(root.descendants())
    .join("text")
      .style("fill-opacity", d => d.parent === root ? 1 : 0)
      .style("display", d => d.parent === root ? "inline" : "none")
      .text(d => d.data.name);

  zoomTo([root.x, root.y, root.r * 2]);

  function zoomTo(v) {
    const k = width / v[2];

    view = v;

    label.attr("transform", d => `translate(${(d.x - v[0]) * k},${(d.y - v[1]) * k})`);
    node.attr("transform", d => `translate(${(d.x - v[0]) * k},${(d.y - v[1]) * k})`);
    node.attr("r", d => d.r * k);
  }

  function zoom(event, d) {
    const focus0 = focus;

    focus = d;

    const transition = svg.transition()
        .duration(event.altKey ? 7500 : 750)
        .tween("zoom", d => {
          const i = d3.interpolateZoom(view, [focus.x, focus.y, focus.r * 2]);
          return t => zoomTo(i(t));
        });

    label
      .filter(function(d) { return d.parent === focus || this.style.display === "inline"; })
      .transition(transition)
        .style("fill-opacity", d => d.parent === focus ? 1 : 0)
        .on("start", function(d) { if (d.parent === focus) this.style.display = "inline"; })
        .on("end", function(d) { if (d.parent !== focus) this.style.display = "none"; });
  }

  return svg.node();
}


function _17(md){return(
md`### Recap

- There is an **unparalleled diversity** of openings played
- **1. d4** and **1. e4** are the most popular starting moves
- **1. d5** and **1. e5** are the most popular responses to d4 and e4, respectively
- **2. Bc4** is the dominant response to the Sicilian Defense **(1. e4 1... c5)**
`
)}

function _18(md){return(
md`## Intermediate (1500+)`
)}

function _19(md){return(
md`### Analysis 

At the intermediate level, openings are still diverse but **1. e4** has overtaken **1. d4** as a more clear favorite for the first move played by White. Additionally, by looking a level deeper under e4, **1... e6**, which is also known as the *French Defense*, has grown to be equally as popular of a response to 1. e4 as **1... d5** in comparison to e5's dominance at the beginner level.

Iterating through the Sicilian Defense **(1. e4 1... c5)**, we find that **2. Bc4** has lost its spot as the #1 response against the Sicilian. Instead, **2. c4** and **2. Nf3** have become the more popular responses, resemblant of changing philosophies regarding developing the bishop. In the Sicilian Defense, the bishop can become a liability on the c4-square over time so it is not surprising to see that we see its reduced frequency in play.

Understanding why it is a liability requires the understanding that Black's pieces are better equipped in the Sicilian setup to attack the c4-bishop.

Another interesting thing to note is the rise of the Indian Defense **(1... Nf6)** against **1. d4** compared to the beginner level. We can see that players are starting to adopt more modern playstyles that capitalize on applying pressure to the center rather than physically occupying it. This leads to *rich, positional games *— characteristic of a newfound positional understanding by intermediate-level players. This is complemented by the growth of **1. Nf3** and **1. c4** as starting moves which also indirectly attack the center.`
)}

function _intermediate(pack,intermediate_data,d3,width,height,color)
{
  const root = pack(intermediate_data);
  let focus = root;
  let view;

  var div = d3.select("body").append("div")
     .style("position", "absolute")
     .style("text-align", "center")
     .style("padding", .5)
     .style("background", "#FFFFFF")
     .style("color", "#313639")
     .style("border", "1px solid #313639")
     .style("border-radius", "8px")
     .style("font-size", "1.3rem")
     .style("opacity", 0);

  const svg = d3.create("svg")
      .attr("viewBox", `-${width / 2} -${height / 2} ${width} ${height}`)
      .style("display", "block")
      .style("margin", "0 -14px")
      .style("background", "black")
      .style("cursor", "pointer")
      .on("click", (event) => zoom(event, root));
  
  const node = svg.append("g")
    .selectAll("circle")
    .data(root.descendants().slice(1))
    .join("circle")
      .attr("fill", d => d.children ? color(d.data.name) : "black")
      .attr("pointer-events", d => !d.children ? "none" : null)
      .on("mouseover", function(d) { d3.select(this).attr("stroke", "#ffffff").attr("stroke-width", 3); 
                                    div.transition().duration('50').style("opacity",1); 
                                    let name = d.data.name;
                                    div.html(name).style("left", (d3.event.pageX + 10) + "px")
                                                  .style("top", (d3.event.pageY - 15) + "px");})
      .on("mouseout", function() { d3.select(this).attr("stroke", null); 
                                   div.transition().duration('50').style("opacity", 0); })
      .on("click", (event, d) => focus !== d && (zoom(event, d), event.stopPropagation()));
    
  node.append("title")
      .text(d => !d.data.children? `${d.data.name}` : `${d.data.name}\nUnique Responses: ${d.data.children.length}`);

  const label = svg.append("g")
      .style("font", "48px sans-serif")
      .attr("fill", "white")
      .attr("pointer-events", "none")
      .attr("text-anchor", "middle")
    .selectAll("text")
    .data(root.descendants())
    .join("text")
      .style("fill-opacity", d => d.parent === root ? 1 : 0)
      .style("display", d => d.parent === root ? "inline" : "none")
      .text(d => d.data.name);

  zoomTo([root.x, root.y, root.r * 2]);

  function zoomTo(v) {
    const k = width / v[2];

    view = v;

    label.attr("transform", d => `translate(${(d.x - v[0]) * k},${(d.y - v[1]) * k})`);
    node.attr("transform", d => `translate(${(d.x - v[0]) * k},${(d.y - v[1]) * k})`);
    node.attr("r", d => d.r * k);
  }

  function zoom(event, d) {
    const focus0 = focus;

    focus = d;

    const transition = svg.transition()
        .duration(event.altKey ? 7500 : 750)
        .tween("zoom", d => {
          const i = d3.interpolateZoom(view, [focus.x, focus.y, focus.r * 2]);
          return t => zoomTo(i(t));
        });

    label
      .filter(function(d) { return d.parent === focus || this.style.display === "inline"; })
      .transition(transition)
        .style("fill-opacity", d => d.parent === focus ? 1 : 0)
        .on("start", function(d) { if (d.parent === focus) this.style.display = "inline"; })
        .on("end", function(d) { if (d.parent !== focus) this.style.display = "none"; });
  }

  return svg.node();
}


function _21(md){return(
md`### Recap

- A large amount of **diverse** openings continue to be played
- **1. e4** is now the clear top-starting move over **1. d4**
- The French Defense **(1. e6)** has become an equally popular response to **1... e5** against **1. e4**
- **2. Bc4** is no longer the dominant response to the Sicilian Defense **(1. e4 1... c5)**`
)}

function _22(md){return(
md`## Advanced (2000+)`
)}

function _23(md){return(
md`At the advanced level, the variety in opening moves shrinks significantly. Moves such as **1. g3** and **1. h4** that had a decent amount of representation at the intermediate level are almost non-existent. Now, there are only four opening moves that are played at a significant: **1. e4, 1. d4**, **1. Nf3**, and **1. c4**. Out of these moves, Nf3 is the most interesting, as its popularity shoots up in comparison to the previous levels and is now comparable to **1. e4** and **1. d4**.

If we delve a level deeper into **1. e4**, we can immediately see how the variety in responses by black has diminished. The total number of responses drops from 19 to 14. We can continue further into **1... c5**, or the Sicilian Defense, and we can see the number of second moves by white has dropped from 21 to 14. This trend can be seen if you follow any popular line and compare the intermediate and advanced levels. 

Looking at the Sicilian Defense **(1. e4 1... c5)**, notice how **2. Bc4** is one of the smallest circles in size. What was the most popular response for beginners against the Sicilian is now one of the most unpreferred responses for advanced players!`
)}

function _advanced(pack,advanced_data,d3,width,height,color)
{
  const root = pack(advanced_data);
  let focus = root;
  let view;

  var div = d3.select("body").append("div")
     .style("position", "absolute")
     .style("text-align", "center")
     .style("padding", .5)
     .style("background", "#FFFFFF")
     .style("color", "#313639")
     .style("border", "1px solid #313639")
     .style("border-radius", "8px")
     .style("font-size", "1.3rem")
     .style("opacity", 0);

  const svg = d3.create("svg")
      .attr("viewBox", `-${width / 2} -${height / 2} ${width} ${height}`)
      .style("display", "block")
      .style("margin", "0 -14px")
      .style("background", "black")
      .style("cursor", "pointer")
      .on("click", (event) => zoom(event, root));
  
  const node = svg.append("g")
    .selectAll("circle")
    .data(root.descendants().slice(1))
    .join("circle")
      .attr("fill", d => d.children ? color(d.data.name) : "black")
      .attr("pointer-events", d => !d.children ? "none" : null)
      .on("mouseover", function(d) { d3.select(this).attr("stroke", "#ffffff").attr("stroke-width", 3); 
                                    div.transition().duration('50').style("opacity",1); 
                                    let name = d.data.name;
                                    div.html(name).style("left", (d3.event.pageX + 10) + "px")
                                                  .style("top", (d3.event.pageY - 15) + "px");})
      .on("mouseout", function() { d3.select(this).attr("stroke", null); 
                                   div.transition().duration('50').style("opacity", 0); })
      .on("click", (event, d) => focus !== d && (zoom(event, d), event.stopPropagation()));
    
  node.append("title")
      .text(d => !d.data.children? `${d.data.name}` : `${d.data.name}\nUnique Responses: ${d.data.children.length}`);

  const label = svg.append("g")
      .style("font", "48px sans-serif")
      .attr("fill", "white")
      .attr("pointer-events", "none")
      .attr("text-anchor", "middle")
    .selectAll("text")
    .data(root.descendants())
    .join("text")
      .style("fill-opacity", d => d.parent === root ? 1 : 0)
      .style("display", d => d.parent === root ? "inline" : "none")
      .text(d => d.data.name);

  zoomTo([root.x, root.y, root.r * 2]);

  function zoomTo(v) {
    const k = width / v[2];

    view = v;

    label.attr("transform", d => `translate(${(d.x - v[0]) * k},${(d.y - v[1]) * k})`);
    node.attr("transform", d => `translate(${(d.x - v[0]) * k},${(d.y - v[1]) * k})`);
    node.attr("r", d => d.r * k);
  }

  function zoom(event, d) {
    const focus0 = focus;

    focus = d;

    const transition = svg.transition()
        .duration(event.altKey ? 7500 : 750)
        .tween("zoom", d => {
          const i = d3.interpolateZoom(view, [focus.x, focus.y, focus.r * 2]);
          return t => zoomTo(i(t));
        });

    label
      .filter(function(d) { return d.parent === focus || this.style.display === "inline"; })
      .transition(transition)
        .style("fill-opacity", d => d.parent === focus ? 1 : 0)
        .on("start", function(d) { if (d.parent === focus) this.style.display = "inline"; })
        .on("end", function(d) { if (d.parent !== focus) this.style.display = "none"; });
  }

  return svg.node();
}


function _25(md){return(
md`### Recap

- The pool of openings has **shrunk** to four main choices: **1. e4**, **1. d4**, **1. Nf3**, and **1. c4**
- The Sicilian Defense **(1. c5)** has become the undisputed response against **1. e4**
- **2. Bc4** is considered an inferior response to the Sicilian Defense **(1. e4 1... c5)**`
)}

function _26(md){return(
md`### Summary 
At first glance, all three experience levels look similar, but it is clear that increasing skill level pertains to a decreasing diversity of plays made. For the beginners, there are 7 extremely frequent first moves, for intermediate players, there are 6, and for advanced players, there are only four extremely frequent first moves. 

The structure of the opening moves is almost fractal in nature, with the beginner level's next moves splitting off into many more variations than the advanced. All of this reveals that there are some better openings than others, and the beginners don't know what they are yet.

Another observation from the data visualization is the increase in using the knight on the first or second move from beginner to advanced. For beginners, Nf3 is tied for 5th most common opening move, while Nf3 is tied for the most common move among advanced players. This is likely because knights have more complicated move set compared to pawns, and it takes experience to become comfortable working with the piece. 

All in all, we find that if you are looking to become a better player, guide yourself through the opening choices at the advanced level and explore which openings have made players successful on the [Lichess platform](https://lichess.org/). 

We hope that you too can become a chess master!`
)}

function _pack(d3,width,height){return(
data => d3.pack()
    .size([width, height])
    .padding(3)
  (d3.hierarchy(data)
    .sum(d => d.value)
    .sort((a, b) => b.value - a.value))
)}

function _beginner_data(FileAttachment){return(
FileAttachment("beginner.json").json()
)}

function _intermediate_data(FileAttachment){return(
FileAttachment("intermediate.json").json()
)}

function _advanced_data(FileAttachment){return(
FileAttachment("advanced@3.json").json()
)}

function _color(d3,beginner_data){return(
d3.scaleOrdinal(d3.quantize(d3.interpolateCividis, beginner_data.children.length + 1))
)}

function _format(d3){return(
d3.format(",d")
)}

function _width(){return(
932
)}

function _height(width){return(
width
)}

function _d3(require){return(
require("d3@6")
)}

export default function define(runtime, observer) {
  const main = runtime.module();
  const fileAttachments = new Map([["intermediate.json",new URL("./files/fec6995852c12f364d7f3325ac7c302c650df551e0a404694454366574aa923834cad58aba4e6bc982a33b35174b43778d4d0b6879c78cf43d33d35ead581a93",import.meta.url)],["beginner.json",new URL("./files/0b37a129c9eea9688ef35274a78162e5d907aadbc759b59bb454017069329c3aa882f7bb243b355c0421b7d12fa2fb38d059dd740675450a6680d520300c426e",import.meta.url)],["advanced@3.json",new URL("./files/03add02c79ae1749e3fee787d805ca134d70ee459979dafe9c49bb3708eefee8577019470e5eae20e83cd7e3176b73ea8665eee6b8b89d8414d09dc0303f5e6c",import.meta.url)],["chess board.jpg",new URL("./files/87069b68deea415542a4aaca71908ec4de475b10fd502ef95645fd502450bb83382ec78a36f6e798926f035b50477331c30ea3b0ec1dafa4a7467c1fdf6e9a6b",import.meta.url)],["Screenshot 2022-03-08 004236.png",new URL("./files/4f33d58bb33851a597a36a9cd137d6f309d4e2f8bccc5d192801844bf168f90465beef1428aeba0b2e8cb1f7f8ce1bd1462fedc7f43b9111b72f976310d1c545",import.meta.url)],["Screenshot 2022-03-08 102817.png",new URL("./files/c3d93fdb496b74e5561b86e6dff04634fcfe998fac35475f8c7da12e59dc9e0a6c4de152d6744c182cb52b3cc2b06a658a68f9c0537e0477d778ffce0700e123",import.meta.url)]]);
  main.builtin("FileAttachment", runtime.fileAttachments(name => fileAttachments.get(name)));
  main.variable(observer()).define(["md"], _1);
  main.variable(observer()).define(["md"], _2);
  main.variable(observer()).define(["md"], _3);
  main.variable(observer()).define(["FileAttachment","md"], _4);
  main.variable(observer()).define(["md"], _5);
  main.variable(observer()).define(["FileAttachment","md"], _6);
  main.variable(observer()).define(["md"], _7);
  main.variable(observer()).define(["md"], _8);
  main.variable(observer()).define(["md"], _9);
  main.variable(observer()).define(["md"], _10);
  main.variable(observer()).define(["md"], _11);
  main.variable(observer()).define(["md"], _12);
  main.variable(observer()).define(["FileAttachment","md"], _13);
  main.variable(observer()).define(["md"], _14);
  main.variable(observer()).define(["md"], _15);
  main.variable(observer("beginner")).define("beginner", ["pack","beginner_data","d3","width","height","color"], _beginner);
  main.variable(observer()).define(["md"], _17);
  main.variable(observer()).define(["md"], _18);
  main.variable(observer()).define(["md"], _19);
  main.variable(observer("intermediate")).define("intermediate", ["pack","intermediate_data","d3","width","height","color"], _intermediate);
  main.variable(observer()).define(["md"], _21);
  main.variable(observer()).define(["md"], _22);
  main.variable(observer()).define(["md"], _23);
  main.variable(observer("advanced")).define("advanced", ["pack","advanced_data","d3","width","height","color"], _advanced);
  main.variable(observer()).define(["md"], _25);
  main.variable(observer()).define(["md"], _26);
  main.variable(observer("pack")).define("pack", ["d3","width","height"], _pack);
  main.variable(observer("beginner_data")).define("beginner_data", ["FileAttachment"], _beginner_data);
  main.variable(observer("intermediate_data")).define("intermediate_data", ["FileAttachment"], _intermediate_data);
  main.variable(observer("advanced_data")).define("advanced_data", ["FileAttachment"], _advanced_data);
  main.variable(observer("color")).define("color", ["d3","beginner_data"], _color);
  main.variable(observer("format")).define("format", ["d3"], _format);
  main.variable(observer("width")).define("width", _width);
  main.variable(observer("height")).define("height", ["width"], _height);
  main.variable(observer("d3")).define("d3", ["require"], _d3);
  return main;
}
